__doc__ = """

	Wizard.py
"""

__history__ = """ (Please put the most recent comment at the top.)

	11 Jul 2008 FlottenKdo PGI1001 Gorden Schollbach
		changed nextPage-function to get an alert-box by multiple clicking of the "next"-button 
		(prevented for multiple equal workflow-item-comments and workflow-state-subscription-mails)
			added the variable "wizard_again" out of the nextPage-function for the alert-box

	10 FEB 2004	Nick Channon
		- Updated to be more thread safe when using sessions

	30 APR 2003	Nick Channon
		- fixed to use new form validation

	14 MAR 2003	Nick Channon
		- fixed the else block so sessionSetGet would be called

	12 MAR 2003	Margie Watkins
		- add cancel button and automatic session variable set and clear

	28 Jan 2003	Jennifer Jeng
		- Added javascript reference for 'wtf_html_editor_js'.

	19 Nov 2002	Margie Watkins
		- fix img/js refs

	30 Jul 2002	Jennifer Jeng
		- Changed 'wizardform' into string. 

	17 Jul 2002	Jennifer Jeng
		- Added Javascript '<script>var eltswizardform = new Array();</script>'
			reference for form validation in '_header' string.
                - Added 'overDiv' JavaScript reference in '_header' string.

	16 Jul 2002	Jennifer Jeng
		- Added Javascript reference for 'universaldate_js' and 'calendar_js'
			in '_header' string so the date picker would popup.
		
	01 Jul 2002	Margie Watkins
		- onclick methods call validateForm method change next to input type=image
			action=formSubmit for wise tag compatibility.

"""

__version__='$Revision$'[11:-2]

import string
import time

from DocumentTemplate.DT_Util import *		# useful parsing methods you'll need 
from DocumentTemplate.DT_String	import String	# explained later 
from WT_Util import *

from DocumentTemplate.DT_String	import String
from DocumentTemplate.DT_Util import Eval, parse_params, str
from random import Random
from thread import allocate_lock
from Products.WISE.WISEFileUpload import USE_UPLOAD_HANDLER

# returns a new wizard id (should be unique between threads)
_r_gen = Random()
_r_gen_lock = allocate_lock()
def newWizardId():
	# acquire lock
	_r_gen_lock.acquire()
	try:
		wiz_id = ("wiz" + str(time.time()) + str(_r_gen.random())).replace('.','')
	finally:
		_r_gen_lock.release()
	return wiz_id

_header='''
<div id="overDiv" style="position:absolute; visibility:hide; z-index: 1;"></div>
<script language="JavaScript" src="wtf_overlib_js"></script>
<script type="text/javascript">
<!--
	var wizard_page=%(page)d
	var wizard_again=true;

	function nextPage()
	{
	if (wizard_again == true)
		{
		var form = document.all ? document.all.wizardform : document.forms.wizardform;
		form.wizard_page.value = wizard_page + 1;
		var inputcheck = validateForm('wizardform');
		if (inputcheck)
		{
			wizard_again = false;
		}
		return inputcheck;
		}
	else
		{
		alert('Die Verarbeitung Ihrer Eingabe luft bereits, bitte haben Sie einen Moment Geduld.'); 
		return false;
		}
	}

	function prevPage()
	{
		if (wizard_page > 1)
		{
			var form = document.all ? document.all.wizardform : document.forms.wizardform;
			form.wizard_page.value = wizard_page - 1;
			//do not do validation if going back a page
			form.submit();
		}
	}

	function cancelWizard()
	{
		var form = document.all ? document.all.wizardform : document.forms.wizardform;
		form.wizard_cancel.value = wizard_cancel = 1;
		//do not do validation if cancelling
		form.submit();
	}
//-->
</script>
<script language='JavaScript' src='wtf_formvalidation_js'></script>
<script language='JavaScript' src='wtf_universaldate_js'></script>
<script language='JavaScript' src='wtf_calendar_js'></script>
<script language='JavaScript' src='wtf_color_js'></script>
<script language='JavaScript' src='wtf_html_editor_js'></script>
<form name="wizardform" action="%(submit)s" method="post" enctype="multipart/form-data">
<script><!--
var eltswizardform = new Array();
var htmleditorwizardform = new Array();
//--></script>
<input type="hidden" name="waction" value="%(waction)s">
<input type="hidden" name="wizard_page" value="%(page)d">
<input type="hidden" name="wizard_cancel" value="0">
<input type="hidden" name="wizard_session" value="%(session)s">
'''

_footer_tmpl = '''<br><center>%s</center><br>
<script><!--
onloadArray[onloadArray.length] = function () {
 formInit('wizardform', eltswizardform); }
//--></script></form>'''

# next, previous and cancel buttons
_next = '<input name="next" value="next" onClick="return nextPage();" type=image border="0" src="wtf_next_button" alt="Next Page">'

_previous = '<a href="javascript:prevPage()"><img border="0" src="wtf_prev_button" alt="Previous Page"></a>'

_cancel = '<a href="javascript:cancelWizard()"><img border="0" src="wtf_cancel_button" alt="Cancel"></a>'

# first page footer
_first_page_footer = _footer_tmpl % _next

# second and higher pages footer
_all_pages_footer= _footer_tmpl % '&nbsp;'.join((_previous,_next,_cancel))

_sessionSetGet = '''sessionSetGet(session='%s')'''
_sessionClear = '''sessionClear(session='%s')'''


#----------------------------------------
#	Wizard
#----------------------------------------
class Wizard:
	''' Wizard Class '''
	blockContinuations=('wnext', 'else')
	name='wwizard'

	def __init__(self, blocks):
		"""build our wizard"""

		tname, args, section = blocks[0]
		self.__name__ = tname
		
		# get the else block
		#
		if blocks[-1][0] == 'else':
			tname, args, section = blocks[-1]
			del blocks[-1]
			args=parse_params(args, name='')
			if args:
				ename,expr=name_param(args,'else',1)
				if ename != name:
					raise ParseError, (
					'name in else does not match wizard', 'in')
			self.else_block = section.blocks
		else:
			self.else_block = None
		
		page_blocks = []

		for tname, args, section in blocks:
			# can't have more than one else block
			if tname == 'else':
				raise ParseError, (
				'more than one else tag for a single wizard tag', 'in')
			# add our block to the pages	
			page_blocks.append(section.blocks)

		self.page_blocks = page_blocks
	
	def render(self, md):
		"""render our wizard"""
		REQUEST = md['REQUEST']
		form  = REQUEST.form
		# get the session id from the form
		try:
			session_id = form['wizard_session']
		except:
			session_id = newWizardId()

		# determine waction for form
		waction = form.get('waction','')
		if not waction:
			# no waction available determine from url
			# we should be submitting to the page that contains
			# the wizard
			waction = REQUEST['URL0'].split('/')[-1]
 
		# get the page number and if we are canceling
		wizard_cancel = int(form.get('wizard_cancel',0))
		wizard_page = int(form.get('wizard_page',0))

		# do a little validation on input
		if wizard_page < 0:
			wizard_page = 0

		html = []

		# are we are canceling
		if wizard_cancel:
			# clear the session and set page number to 0
			Eval(_sessionClear % session_id).eval(md)
			wizard_page = 0
		else:
			# perform a session set get
			Eval(_sessionSetGet % session_id).eval(md)

		# set values used in the header
		html_vars = {'session': session_id,
				'page': wizard_page,
				'waction': waction}

		# are we are done
		if wizard_page >= len(self.page_blocks):
			# render else block
			html.append(render_blocks(self.else_block, md))
			# clear session
			Eval(_sessionClear % session_id).eval(md)
		else:
			# render the wizard contents
			page_block = self.page_blocks[wizard_page]
			# will hold a list of file upload field names
			files_list = []
			# render main body of page
			md._push({'form_name': 'wizardform',
					'_wtf_files_list':files_list})

			page_content = render_blocks(page_block, md)

			# determine if any files were used
			# and use the apache uploaded in required
			if USE_UPLOAD_HANDLER and files_list:
				html_vars['submit'] = '_wf_upload'
			else:
				html_vars['submit'] = 'formSubmit'

			md._pop()

			# render the header
			html.append(_header % html_vars)
			# add the page
			html.append(page_content)
			# is this the first page
			if wizard_page:
				html.append(_all_pages_footer)
			else:
				html.append(_first_page_footer)

		# return the final page
		return ''.join(html)
		
	__call__ = render
# Add our tag to DocumentTemplate known tags !
# commands is a dictionary indexed by tag names DocumentTemplate knows.

String.commands['wwizard'] = Wizard

